__doc__ = """

   MonkeyItem is an item generated by a MonkeyMan.

   """

__version__ = 0.1

ADD_CONTENT_PERMISSION = "Add portal content"

from AccessControl import ClassSecurityInfo, getSecurityManager

from Products.CMFCore import CMFCorePermissions
from Products.CMFDefault.Link import Link

# PortalContent classes need to be registered.
# This installs the class's factory method and icon.
from Products.CMFCore.register import registerPortalContent
registerPortalContent('MonkeyItem',
                      constructors=('addMonkeyItem',),
                      action='monkeyitem_add_form',
                      icon="monkeyitem_icon.png",
                      productGlobals=globals()
                     )

factory_type_information = { 'id'              : 'MonkeyItem'
                            , 'title'          : 'MonkeyItem'
                            , 'meta_type'      : 'MonkeyItem' 
                            , 'description'    : ('An item created by a MonkeyMan.')
                            , 'icon'           : 'monkeyitem_icon.png'
                            , 'product'        : 'CMFMonkeyMan'
                            , 'factory'        : 'addMonkeyItem'
                            , 'immediate_view' : 'monkeyitem_edit_form'
                            , 'allow_discussion' : 1
                            , 'actions'        :
                                ( { 'name'          : 'View'
                                  , 'action'        : 'monkeyitem_view'
                                  , 'permissions'   : ('View',)
                                  , 'category'      : 'object'
                                  }
                                , { 'name'          : 'Edit'
                                  , 'action'        : 'monkeyitem_edit_form'
                                  , 'permissions'   : ('Modify portal content',)
                                  , 'category'      : 'object'
                                  }
                                , { 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : ('Modify portal content',)
                                  , 'category'      : 'object'
                                  }
                                )
                            ,
                           }

def addMonkeyItem( self
                , id
                , irc_id = 0
                , irc_channel = ''
                , title = ''
                , remote_url = ''
                , description = ''
                ):
    """Add a MonkeyItem"""
    
    obj=MonkeyItem( id, irc_id, irc_channel, title, remote_url, description )
    self._setObject( id, obj )

class MonkeyItem( Link ):
    """
    An item created by a MonkeyMan.
    """

    portal_type = meta_type='MonkeyItem'

    # declarative security
    security = ClassSecurityInfo()

    def __init__( self
                , id
                , irc_id = 0
                , irc_channel = ''
                , title=''
                , remote_url=''
                , description=''
                ):
        """initialize a new instance of a MonkeyItem"""
        Link.__init__( self, id, title, remote_url, description )
        self.irc_channel = irc_channel
        self.irc_id = irc_id
        self.ref_letter = None

    security.declareProtected(CMFCorePermissions.View, 'refLetter')
    def refLetter(self):
       """
       If item is one of the 26 most recently posted, return it's reference letter
       """
       return self.ref_letter
