__doc__ = """

    MonkeyMan is The Man.

    """

__version__ = 0.1

ADD_CONTENT_PERMISSION = "Add portal content"


from AccessControl import ClassSecurityInfo, getSecurityManager
from DateTime import DateTime
import string
import re
import urllib
import types

from TitleHTMLParser import TitleHTMLParser

from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from Products.CMFCore.utils import getToolByName
from Products.CMFDefault.DiscussionItem import addDiscussionItem

# PortalContent classes need to be registered.
# This installs the class's factory method and icon.
from Products.CMFCore.register import registerPortalContent
registerPortalContent('MonkeyMan',
                        constructors=('addMonkeyMan',),
                        action='monkeyman_add_form',
                        icon="monkeyman_icon.png",
                        productGlobals=globals()
                        )

factory_type_information = { 'id'              : 'MonkeyMan'
                            , 'meta_type'      : 'MonkeyMan' 
                            , 'description'    : ('A Manager of comments recieved from an IRC channel.')
                            , 'icon'           : 'monkeyman_icon.png'
                            , 'product'        : 'CMFMonkeyMan'
                            , 'factory'        : 'addMonkeyMan'
                            , 'immediate_view' : 'monkeyman_edit_form'
                            , 'implicitly_addable' : 0
                            , 'actions'        :
                                ( { 'name'          : 'View'
                                    , 'action'        : 'monkeyman_view'
                                    , 'permissions'   : ('View',)
                                    , 'category'      : 'object'
                                    }
                                , { 'name'          : 'Edit'
                                    , 'action'        : 'monkeyman_edit_form'
                                    , 'permissions'   : ('Modify portal content',)
                                    , 'category'      : 'object'
                                    }
                                , { 'name'          : 'Metadata'
                                    , 'action'        : 'metadata_edit_form'
                                    , 'permissions'   : ('Modify portal content',)
                                    , 'category'      : 'object'
                                    }
                                )
                            ,
                            }

def addMonkeyMan( self
                , id
                , title=''
                , description=''
                ):
    """Add a MonkeyMan"""
    
    obj=MonkeyMan( id, title, description )
    self._setObject( id, obj )

        
class MonkeyMan( PortalContent, DefaultDublinCoreImpl ):
    """
    A manager of items recieved from an IRC channel.
    """

    __implements__ = ( PortalContent.__implements__
                        , DefaultDublinCoreImpl.__implements__
                        )

    portal_type = meta_type='MonkeyMan'

    # declarative security
    security = ClassSecurityInfo()

    def __init__( self
                , id
                , title=''
                , description=''
                ):
        """initialize a new instance of a MonkeyMan"""
        self.id = id
        self.help_url = "http://www.diaweblog.org/COMMANDS"
        self.language = "en" # only used for RSS
        self.notify_weblogs_com = 0 # Notify http://www.weblogs.com
        self.irc_server = ''
        self.irc_channel = ''
        self.irc_nick = ''
        self.irc_username = ''
        self.irc_nickserv = ''      # IRC username
        self.irc_nickserv_pass = '' # IRC password
        self.irc_greeting = ''
        self.ignore_prefixs = ['~']  # URLs prefixed by these characters are ignored by MonkeyMan
        self.ignores = [] # nicknames of IRC users to ignore
        self.shortnick = '@' # commands to MonkeyMan can be prefixed by this instead of the lengthier "MonkeyMan: "
        self.anonymous_account_username = 'monkeymen' # CMF account id that gets used if the user doesn't exist on the system
        self.irc_posting_foldername = 'irc_postings' # Folder that MonkeyItems are placed in
        self.irc2zope = {} # translates IRC nicknames into Zope usernames
        self.recent_post_refs = {} # key
        self.last_ref_id = 25
        self._proxy_roles = ('Member','Owner','Manager')
        self.irc_id_count = 0
        DefaultDublinCoreImpl.__init__( self, title, description )

        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit( self
            , title=''
            , description=''
            , help_url = ''
            , language = ''
            , notify_weblogs_com = 0
            , irc_server = ''
            , irc_channel = ''
            , irc_nick = ''
            , irc_username = ''
            , irc_nickserv = ''
            , irc_nickserv_pass = ''
            , irc_greeting = ''
            , ignore_prefixs = []
            , ignores = []
            , shortnick = ''
            , anonymous_account_username = ''
            , irc_posting_foldername = ''
            , REQUEST = None
            ):
        """
        Edit a MonkeyMan
        """
        self.title = title
        self.description = description
        self.help_url = help_url
        self.language = language
        self.notify_weblogs_com = notify_weblogs_com
        self.irc_server = irc_server
        self.irc_channel = irc_channel
        self.irc_nick = irc_nick
        self.irc_username = irc_username
        self.irc_nickserv = irc_nickserv
        self.irc_nickserv_pass = irc_nickserv_pass
        self.irc_greeting = irc_greeting
        self.ignore_prefixs = ignore_prefixs
        self.ignores = ignores
        self.shortnick = shortnick
        self.anonymous_account_username
        self.irc_posting_foldername = irc_posting_foldername
        if REQUEST is not None:
            count = 0
            for item in REQUEST.ircnames:
                self.irc2zope[item] = REQUEST.zopenames[count]
                count = count + 1

    
    security.declareProtected(CMFCorePermissions.AddPortalContent, 'addMonkeyItem')
    def addMonkeyItem(self, irc_username, title='', remote_url='', description='', modification_date=None, need_path=0):
        """
        Add a MonkeyItem
        """
        pm = getToolByName(self, 'portal_membership')

        irc_username = self._convertUsername( irc_username )
        if not pm.getMemberById( irc_username ):
            irc_username = self.anonymous_account_username
            
        home_dir = pm.getHomeFolder( irc_username )
        owner = home_dir.getOwner()
        user = owner.__of__(self.acl_users)
    
        if not self.irc_posting_foldername in home_dir.objectIds():
            # 'irc_postings' Folder doesn't exist, so create it
            home_dir.manage_addProduct['CMFCore'].manage_addContent(id=self.irc_posting_foldername, type='Portal Folder')
            home_dir[self.irc_posting_foldername].changeOwnership(user)
            
        if title == '' and remote_url != '':
            title = self._fetchTitleFromUrl(remote_url)
            if not title:
                # oops, we couldn't fetch the title
                # this most likely means that we were given a bad URL we'd better bail
                return "Could not fetch title from URL, perhaps you used a bad URL?"
            
        irc_dir = home_dir[self.irc_posting_foldername]
        id = self._generateId(self.meta_type)
        irc_dir.invokeFactory('MonkeyItem', id)
        monkeyitem = irc_dir[id]
        monkeyitem.irc_id = self.irc_id_count
        self.irc_id_count = self.irc_id_count + 1
        monkeyitem.irc_channel = self.irc_channel
        monkeyitem.setDescription( description )
        monkeyitem.edit( remote_url )
        monkeyitem.changeOwnership(user)
        monkeyitem.setTitle( title )

        # None out the oldest MonkeyItems ref_letters
        c = self.last_ref_id + 1
        if c == 26: c = 0
        if self.recent_post_refs.has_key( chr(97 + c) ):
            mi = self.getPhysicalRoot().unrestrictedTraverse( self.recent_post_refs[chr(97 + c)] )
            mi.ref_letter = None
        
        # get an ID for the object an assign it to the [a-z] list
        self.last_ref_id = self.last_ref_id + 1
        if self.last_ref_id >= 26:
            self.last_ref_id = 0
        self.recent_post_refs[ chr(97 + self.last_ref_id) ] = irc_dir[id].getPhysicalPath()

        # update the 26 most recent MonkeyItems ref_letters
        for k,v in self.recent_post_refs.items():
            i = self.getPhysicalRoot().unrestrictedTraverse( v )
            i.ref_letter = k

        if modification_date:
            irc_dir[id].creation_date = self._datify(modification_date)
        irc_dir[id].reindexObject()

        if need_path:
            return irc_dir[id].getPhysicalPath()

        # check if URL already exists
        if self.urlExists(monkeyitem.remote_url):
            self.portal_workflow.doActionFor( monkeyitem, 'hide' )
            return self._viewMonkeyItemText(monkeyitem) + " [hidden - URL already exists]"
        else:
            return self._viewMonkeyItemText(monkeyitem)


    security.declareProtected(CMFCorePermissions.AddPortalContent, 'addComment')
    def addComment(self, irc_username, ref='', title='', description='', modification_date=0):
        """
        Add a Comment
        """
        pm = getToolByName(self, 'portal_membership')
        pd = getToolByName(self, 'portal_discussion')

        irc_username = self._convertUsername( irc_username )
        if not pm.getMemberById( irc_username ):
            irc_username = self.anonymous_account_username
            
        owner = pm.getHomeFolder( irc_username ).getOwner()
        user = pm.wrapUser(owner)

        monkeyitem = self._getMonkeyItem( ref )

        talkback = pd.getDiscussionFor( monkeyitem )
        id = talkback.createReply(title, description, Creator=irc_username)
        if modification_date:
            talkback.getReply(id).creation_date = self._datify(modification_date)


    security.declareProtected(CMFCorePermissions.View, 'viewMonkeyItem')
    def viewMonkeyItem( self, ref):
        """
        View a MonkeyItem (suitable for IRC)
        """
        monkeyitem = self._getMonkeyItem(ref)
        return self._viewMonkeyItemText(monkeyitem)


    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'hideMonkeyItem')
    def hideMonkeyItem( self, ref):
        """
        Hides a MonkeyItem
        """
        monkeyitem = self._getMonkeyItem(ref)
        if monkeyitem is not None:
            self.portal_workflow.doActionFor( monkeyitem, 'hide' )

            
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'showMonkeyItem')
    def showMonkeyItem( self, ref):
        """
        Shows a hidden MonkeyItem
        """
        monkeyitem = self._getMonkeyItem(ref)
        if monkeyitem is not None:
            self.portal_workflow.doActionFor( monkeyitem, 'show' )


    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'searchMonkeyItemText')
    def searchMonkeyItemText( self, SearchableText, batch=5 ):
        """
        Show a list of items that match some text
        """
        list = []
        if batch < 1:
            batch = 1
        for r in self.portal_catalog.searchResults(SearchableText=SearchableText)[0:batch]:
            list.append( self._viewMonkeyItemText( r.getObject(r.data_record_id_) ) )
        return list
    
    
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setMonkeyItemTitle')
    def setMonkeyItemTitle( self, ref, title=''):
        """
        Set the Title of a MonkeyItem
        """
        monkeyitem = self._getMonkeyItem(ref)
        if monkeyitem is not None:
            monkeyitem.setTitle(title)
            monkeyitem.reindexObject()

            
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setMonkeyItemURL')
    def setMonkeyItemURL( self, ref, url=''):
        """
        Set the URL of a MonkeyItem
        """
        monkeyitem = self._getMonkeyItem(ref)
        if monkeyitem is not None:
            monkeyitem.edit( url )
            monkeyitem.reindexObject()


    security.declareProtected(CMFCorePermissions.AccessContentsInformation, 'getConnectionInfo')
    def getConnectionInfo( self ):
        """
        Return the server connection information in a tuple of the form:
        (server, port, channel, nick)
        """
        s = string.split(self.irc_server, ":", 1)
        server = s[0]
        port = 6667
        if len(s) == 2:
            try:
                port = int(s[1])
            except ValueError:
                raise ValueError
        return (server, port, self.irc_channel, self.irc_nick)

    
    security.declareProtected(CMFCorePermissions.AccessContentsInformation, 'getConnectionInfo')
    def getBehaviourInfo(self):
        """
        return a tuple of bot behaviourial-related attributes:
        (ignores, ignore_prefixs)
        """
        ignores = []
        for item in self.ignores:
            if item != '':
                ignores.append(item)
        ignore_prefixs = []
        for item in self.ignore_prefixs:
            if item != '':
                ignore_prefixs.append(item)
        return (ignores,ignore_prefixs,self.help_url)

    
    security.declareProtected(CMFCorePermissions.View, 'getLastMonkeyItemId')
    def getLastMonkeyItems(self, batch=5):
        """
        Return the most irc_id of the most recently created MonkeyItem
        """
        list = []
        if batch < 1:
            batch = 1
        for r in self.portal_catalog.searchResults(meta_type='MonkeyItem'
                                                    , review_state='visible'
                                                    , sort_on='created'
                                                    , sort_order='reverse')[0:batch]:
            list.append( self._viewMonkeyItemText( r.getObject(r.data_record_id_) ) )    
        return list
    
    security.declareProtected(CMFCorePermissions.View, 'urlExists')
    def urlExists(self, url):
        """
        A useful query for preventing posting of duplicate URLs
        """
        for r in self.portal_catalog.searchResults( meta_type='MonkeyItem', remote_url=url ):
            return 1
        return 0
        
    #
    # MonkeyMan draws the curtain for a little privacy right here
    #
    def _convertUsername( self, irc_username ):
        """
        Convert an irc username into a Zope one using a list of set of mappings
        """
        if self.irc2zope.has_key( irc_username ):
            return self.irc2zope[irc_username]
        else:
            return irc_username

                    
    def _generateId(self, type):
        now = DateTime()
        name = type.replace(' ', '')+'.'+now.strftime('%Y-%m-%d')+'.'+now.strftime('%H%M%S')

        # Reduce chances of an id collision (there is a very small chance that somebody will
        # create another object during this loop)
        base_name = name
        objectIds = self.getParentNode().objectIds()
        i = 1
        while name in objectIds:
            name = base_name + "-" + str(i)
            i = i + 1
        return name

    
    def _getMonkeyItem(self, ref):
        """
        return a MonkeyItem object which is referenced by a MonkeyMan id
        """
        if isinstance(ref, types.TupleType):
            # physical path
            return self.getPhysicalRoot().unrestrictedTraverse( ref )
        elif isinstance(ref, types.ListType):
            return self.getPhysicalRoot().unrestrictedTraverse( tuple(ref) )
        elif re.match('^[a-z]$',ref):
            # letter
            return self.getPhysicalRoot().unrestrictedTraverse( self.recent_post_refs[ref] )
        elif re.match('^\d+$',ref):
            # digit(s)
            for r in self.portal_catalog.searchResults(meta_type='MonkeyItem'):
                if int(r.irc_id) == int(ref):
                    return r.getObject(r.data_record_id_)
            raise "MonkeyReferenceError"
        else:
            raise "MonkeyReferenceError"

        
    def _fetchTitleFromUrl(self, url):
        """
        Fetch the page referenced by an URL and parse out it's title
        """
        title = ''
        try:
            f = urllib.urlopen( url )
            p = TitleHTMLParser()
            for line in f.readlines():
                p.feed(line)
                if p.title:
                    return p.title
            p.close()
            title = p.title
        except Exception, e:
            print "error: %s" % e
            title = None
        return title
        
    def _viewMonkeyItemText(self, monkeyitem):
        """
        Return a text view of a MonkeyItem
        """
        # this whole "t = t + " bit may seem cumbersome but it's got a nice ring to it
        # so please do not remove lest a Dictator's evil henchman shall force ye
        # to drink a whole bottle of Hagen Flea & Tick Shampoo, verily!
        t = ''
        mpath = monkeyitem.absolute_url(relative=1)
        for k,v in  self.recent_post_refs.items():
            if mpath == v:
                t = t + str(k) + ': '
        t = t + monkeyitem.Title()[0:100]
        if len(monkeyitem.Title()) > 100:
            t = t + "..."
        if monkeyitem.remote_url:
            t = t + ' ('
            t = t + monkeyitem.remote_url[0:100]
            if len(monkeyitem.remote_url) > 100:
                t = t + "..."
            t = t + ') '
        t = t + '[' + str(monkeyitem.irc_id) + ']'
        if self.portal_workflow.getInfoFor(monkeyitem,'review_state') == 'private':
            t = t + ' [hidden]'        
        return t
